mod menu;

use sdl3::event::Event;
use sdl3::keyboard::Keycode;
use sdl3::pixels::{Color, PixelFormat, PixelMasks};
use sdl3::rect::Rect;
use sdl3::render::{FRect, WindowCanvas};
use sdl3::surface::Surface;
use sdl3::sys::pixels::{SDL_PixelFormat, SDL_PIXELFORMAT_ARGB8888};
use sdl3::EventPump;
use std::time::Duration;

pub struct UIWindow {
    title: &'static str,
    width: u32,
    height: u32,
    refresh_rate: u32,
    canvas: WindowCanvas,
    event_pump: EventPump,
}

impl UIWindow {
    pub fn new(title: &'static str, width: u32, height: u32, refresh_rate: u32) -> Self {
        let sdl_context = sdl3::init().unwrap();
        let video_subsystem = sdl_context.video().unwrap();

        let window = video_subsystem
            .window(title, width, height)
            .position_centered()
            .build()
            .unwrap();

        let canvas = window.into_canvas();
        let event_pump = sdl_context.event_pump().unwrap();

        Self {
            title,
            width,
            height,
            refresh_rate,
            canvas,
            event_pump,
        }
    }

    pub fn render(&mut self) {
        self.canvas.set_draw_color(Color::RGB(0, 0, 0));
        self.canvas.clear();

        // let test = unsafe { PixelFormat::from_ll(SDL_PIXELFORMAT_ARGB8888) };
        let test = PixelFormat::try_from(SDL_PIXELFORMAT_ARGB8888).unwrap();

        let texture_creator = self.canvas.texture_creator();
        let mut texture = texture_creator
            .create_texture_static(
                test,
                2,
                1,
            )
            .unwrap();
        let color_buffer = vec![0x0, 0x0, 0xff, 0xff, 0x0, 0xff, 0x0, 0xff];
        texture.update(None, &color_buffer, 4).unwrap();

        self.canvas.copy(&texture, None, Rect::new(10, 10, 50, 50)).unwrap();

        self.canvas.present();
    }

    /// Poll and processes events.
    ///
    /// # Returns
    /// `true` if there was a "Quit" event (meaning the user want to close the window)
    pub fn poll_events(&mut self) -> bool {
        for event in self.event_pump.poll_iter() {
            match event {
                Event::Quit { .. }
                | Event::KeyDown {
                    keycode: Some(Keycode::Escape),
                    ..
                } => return true,
                _ => {}
            }
        }

        false
    }

    pub fn wait(&self) {
        ::std::thread::sleep(Duration::new(0, 1_000_000_000u32 / self.refresh_rate));
    }
}

pub trait UIComponent {}
